"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.check = void 0;
const node_path_1 = __importDefault(require("node:path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lib_1 = require("../lib");
const config_1 = require("./config");
const utils_1 = require("./utils");
const STATS = {
    processed: 0,
    excluded: 0,
    otherIssues: 0,
    styleIssues: 0,
};
async function check(options) {
    if (options._.length < 1) {
        process.stderr.write('Expect at least 1 file or directory.\n');
        process.exit(1);
    }
    const config = (0, config_1.loadBaseConfig)(options);
    await Promise.all(options._.map(filePath => processFilePath(filePath, config, options)));
    summary();
}
exports.check = check;
async function processFilePath(filePath, config, options) {
    if (!(await fs_extra_1.default.pathExists(filePath))) {
        STATS.otherIssues++;
        process.stderr.write(`'${filePath}' doesn't exist.\n`);
        return;
    }
    const stat = await fs_extra_1.default.stat(filePath);
    if (stat.isFile())
        await processFile(config, filePath);
    else if (stat.isDirectory())
        await processDirectory(options, config, filePath);
    else {
        STATS.otherIssues++;
        process.stderr.write(`'${filePath}' is neither file nor directory.\n`);
    }
}
async function processFile(baseConfig, filePath, realPath) {
    if (!(0, config_1.isSupported)(filePath))
        return;
    STATS.processed++;
    const resolvedPath = realPath ?? node_path_1.default.resolve(filePath);
    const config = (0, lib_1.resolveConfigForFile)(resolvedPath, baseConfig);
    if ((0, lib_1.isFileExcludedByConfig)(resolvedPath, config)) {
        STATS.excluded++;
        return;
    }
    const source = await fs_extra_1.default.readFile(resolvedPath, { encoding: 'utf8' });
    const result = await (0, lib_1.formatSourceFromFile)(source, resolvedPath, config);
    if (result !== undefined) {
        STATS.styleIssues++;
        process.stderr.write(`'${filePath}' is different after formatting.\n`);
    }
}
async function processDirectory(options, config, dirPath) {
    const { recursive } = options;
    for await (const { relativePath, resolvedPath } of (0, utils_1.getFiles)(dirPath, !recursive)) {
        const filePath = node_path_1.default.join(dirPath, relativePath);
        await processFile(config, filePath, resolvedPath);
    }
}
function summary() {
    const { processed, excluded, otherIssues, styleIssues } = STATS;
    if (processed) {
        process.stdout.write(`Checked ${processed} ${processed === 1 ? 'file' : 'files'}, of which:\n`);
        const passed = processed - excluded - styleIssues;
        if (passed > 0)
            process.stdout.write(`  ${passed} ${passed === 1 ? 'file' : 'files'} passed.\n`);
        if (excluded > 0)
            process.stdout.write(`  ${excluded} ${excluded === 1 ? 'file was' : 'files were'} excluded.\n`);
        if (styleIssues > 0)
            process.stdout.write(`  ${styleIssues} ${styleIssues === 1 ? 'file has' : 'files have'} formatting issues.\n`);
    }
    else
        process.stdout.write('Checked 0 files.\n');
    if (styleIssues + otherIssues > 0)
        process.exit(1);
}
//# sourceMappingURL=check.js.map