"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = void 0;
const node_path_1 = __importDefault(require("node:path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const lib_1 = require("../lib");
const config_1 = require("./config");
const utils_1 = require("./utils");
var OutputMode;
(function (OutputMode) {
    OutputMode[OutputMode["NORMAL"] = 0] = "NORMAL";
    OutputMode[OutputMode["DRY_RUN_FILE"] = 1] = "DRY_RUN_FILE";
    OutputMode[OutputMode["DRY_RUN_FILES"] = 2] = "DRY_RUN_FILES";
    OutputMode[OutputMode["DRY_RUN_DIR"] = 3] = "DRY_RUN_DIR";
})(OutputMode || (OutputMode = {}));
function dryRunOutput(mode, result, source, inputFile) {
    if (mode === OutputMode.DRY_RUN_FILES) {
        if (result !== undefined && inputFile)
            process.stdout.write(`'${inputFile}' will be modified.\n`);
    }
    else if (mode === OutputMode.DRY_RUN_FILE)
        process.stdout.write(result ?? source);
}
async function checkFileContent(filePath, text) {
    const exist = await fs_extra_1.default.pathExists(filePath);
    if (!exist)
        return { exist };
    const stat = await fs_extra_1.default.stat(filePath);
    const isFile = stat.isFile();
    if (!isFile)
        return { exist, isFile };
    if (stat.size !== text.length)
        return { exist, isFile, equal: false };
    const content = await fs_extra_1.default.readFile(filePath, { encoding: 'utf8' });
    return { exist, isFile, equal: text === content };
}
async function outputResult(mode, result, outputFile, source, inputFile) {
    if (outputFile) {
        const text = result ?? source;
        const { exist, isFile, equal } = await checkFileContent(outputFile, text);
        if (exist && !isFile) {
            process.stderr.write(`Option output: '${outputFile}' is not a file.\n`);
            return { error: true };
        }
        if (mode === OutputMode.NORMAL) {
            if (!equal)
                await fs_extra_1.default.outputFile(outputFile, text);
        }
        else
            dryRunOutput(mode, result, source);
        return exist ? { modified: equal ? 0 : 1 } : { created: 1 };
    }
    else if (inputFile) {
        if (mode === OutputMode.NORMAL) {
            if (result !== undefined)
                await fs_extra_1.default.writeFile(inputFile, result);
        }
        else
            dryRunOutput(mode, result, source, inputFile);
        return { modified: result !== undefined ? 1 : 0 };
    }
    else
        dryRunOutput(OutputMode.DRY_RUN_FILE, result, source);
    return {};
}
async function ensureOutputDir(output, dryRun) {
    if (!output)
        return;
    if (!(await fs_extra_1.default.pathExists(output))) {
        if (!dryRun)
            await fs_extra_1.default.mkdir(output, { recursive: true });
    }
    else if (!(await (0, utils_1.isDirectory)(output))) {
        process.stderr.write(`Option output: '${output}' is not a directory.\n`);
        process.exit(1);
    }
}
async function processStdin(options) {
    const { output, dryRun } = options;
    const baseConfig = (0, config_1.loadBaseConfig)(options);
    const chunks = [];
    process.stdin.on('data', data => chunks.push(data.toString()));
    process.stdin.on('end', async () => {
        const source = chunks.join();
        if (!source)
            return;
        const config = (0, lib_1.resolveConfigForSource)(source, baseConfig);
        const ext = (0, config_1.decideExtension)(options);
        const result = await (0, lib_1.formatSourceWithoutFile)(source, ext, config, {
            skipEslintConfig: true,
            skipTsConfig: true,
        });
        const mode = dryRun ? OutputMode.DRY_RUN_FILE : OutputMode.NORMAL;
        const { error, modified, created } = await outputResult(mode, result, output, source);
        if (error)
            process.exit(1);
        if (mode === OutputMode.NORMAL && output)
            summary(mode, modified ?? 0, created ?? 0);
    });
}
async function processDirectory(dirPath, options) {
    if (!fs_extra_1.default.statSync(dirPath).isDirectory())
        return processFiles([dirPath], options);
    const { output, recursive, dryRun } = options;
    const baseConfig = (0, config_1.loadBaseConfig)(options);
    await ensureOutputDir(output, dryRun);
    const mode = dryRun ? OutputMode.DRY_RUN_DIR : OutputMode.NORMAL;
    let modified = 0;
    let created = 0;
    for await (const { relativePath, resolvedPath: inputFile } of (0, utils_1.getFiles)(dirPath, !recursive)) {
        if (!(0, config_1.isSupported)(relativePath))
            continue;
        const filePath = node_path_1.default.join(dirPath, relativePath);
        const config = (0, lib_1.resolveConfigForFile)(inputFile, baseConfig);
        if ((0, lib_1.isFileExcludedByConfig)(inputFile, config))
            continue;
        const source = await fs_extra_1.default.readFile(inputFile, { encoding: 'utf8' });
        const result = await (0, lib_1.formatSourceFromFile)(source, inputFile, config);
        const outputFile = output ? node_path_1.default.join(output, relativePath) : output;
        const { error, modified: m, created: c, } = await outputResult(mode, result, outputFile, source, filePath);
        if (error)
            process.exit(1);
        if (m)
            modified += m;
        if (c)
            created += c;
    }
    summary(mode, modified, created);
}
async function processFiles(filePaths, options) {
    const { output, dryRun } = options;
    const single = filePaths.length === 1;
    if (!single && output) {
        process.stderr.write(`Option output: should be empty if multiple files are provided.`);
        process.exit(1);
    }
    for (const f of filePaths) {
        if (fs_extra_1.default.statSync(f).isFile())
            continue;
        process.stderr.write(`Option: '${f}' is not a file.`);
        process.exit(1);
    }
    const baseConfig = (0, config_1.loadBaseConfig)(options);
    const mode = dryRun
        ? single
            ? OutputMode.DRY_RUN_FILE
            : OutputMode.DRY_RUN_FILES
        : OutputMode.NORMAL;
    const stats = { modified: 0, created: 0 };
    await Promise.all(filePaths.map(filePath => processFile(filePath, baseConfig, output, mode, stats)));
    summary(mode, stats.modified, stats.created);
}
async function processFile(filePath, baseConfig, output, mode, stats) {
    if (!(0, config_1.isSupported)(filePath)) {
        process.stdout.write(`'${filePath}' is not a supported file type.\n`);
        return;
    }
    const inputFile = node_path_1.default.resolve(filePath);
    const config = (0, lib_1.resolveConfigForFile)(inputFile, baseConfig);
    if ((0, lib_1.isFileExcludedByConfig)(inputFile, config)) {
        process.stdout.write(`'${filePath}' is excluded by config.\n`);
    }
    else {
        const source = await fs_extra_1.default.readFile(inputFile, { encoding: 'utf8' });
        const result = await (0, lib_1.formatSourceFromFile)(source, inputFile, config);
        const outputFile = output && (await fs_extra_1.default.pathExists(output)) && (await (0, utils_1.isDirectory)(output))
            ? node_path_1.default.resolve(output, node_path_1.default.basename(inputFile))
            : output;
        const { error, modified, created } = await outputResult(mode, result, outputFile, source, filePath);
        if (error)
            process.exit(1);
        if (modified)
            stats.modified += modified;
        if (created)
            stats.created += created;
    }
}
function summary(mode, modified, created) {
    if (mode === OutputMode.DRY_RUN_FILE || mode === OutputMode.DRY_RUN_FILES)
        return;
    const tense = mode === OutputMode.NORMAL ? '' : 'will be ';
    if (modified || !created)
        process.stdout.write(sumResult(modified, tense + 'modified'));
    if (created)
        process.stdout.write(sumResult(created, tense + 'created'));
}
function sumResult(num, action) {
    return `${num === 0 ? 'No' : num} ${num === 1 ? 'file' : 'files'} ${action}.\n`;
}
async function format(options) {
    switch (options._.length) {
        case 0:
            await processStdin(options);
            break;
        case 1:
            await processDirectory(options._[0], options);
            break;
        default:
            await processFiles(options._, options);
    }
}
exports.format = format;
//# sourceMappingURL=format.js.map