"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfigFromJsonFile = exports.enhanceEol = exports.loadImportSorterConfig = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const condition_1 = require("@dozerg/condition");
const end_of_line_1 = require("@dozerg/end-of-line");
const find_up_1 = __importDefault(require("@dozerg/find-up"));
const common_1 = require("../common");
const merge_1 = require("./merge");
const prettier_1 = require("./prettier");
/**
 * Resolve config for a source file.
 *
 * The following sources will be considered if found (in precedence from high to low):
 * - [ESLint configuration](https://eslint.org/docs/user-guide/configuring)
 * - `"importSorter"` section in `package.json`
 * - `import-sorter.json` (File name is configurable from the base config)
 * - [Prettier configuration](https://github.com/prettier/prettier-vscode#configuration)
 * - `.editorconfig`
 * - The base config provided as parameter
 *
 * @typeparam T - A type extended from Configuration
 *
 * @param fileName - Source file name
 * @param config - Base config
 */
function loadImportSorterConfig(fileName, config) {
    const cfg = config ?? {};
    const pretConfig = (0, prettier_1.loadPretConfig)(fileName);
    const cfgFileName = cfg.configurationFileName || 'import-sorter.json';
    const fConfig = fileConfig(cfgFileName, fileName);
    const pkgConfig = packageConfig(fileName);
    const c = enhanceEol(cfg, () => (0, end_of_line_1.endOfLineForFile)(fileName));
    return (0, merge_1.mergeConfig)(c, pretConfig, fConfig, pkgConfig);
}
exports.loadImportSorterConfig = loadImportSorterConfig;
function fileConfig(fileName, path) {
    const log = (0, common_1.logger)('format-imports.fileConfig');
    log.debug('Loading JSON config from', fileName);
    const files = find_up_1.default.sync(fileName, { type: 'file', cwd: path });
    return readConfigTilRoot(files, file => {
        const config = loadConfigFromJsonFile(file);
        log.debug('Found JSON file', file, 'and config:', config);
        return config;
    });
}
function packageConfig(fileName) {
    const log = (0, common_1.logger)('format-imports.packageConfig');
    log.debug('Loading package.json config for fileName:', fileName);
    const files = find_up_1.default.sync('package.json', { cwd: fileName });
    return readConfigTilRoot(files, file => {
        log.debug('Found package.json in', file);
        const { importSorter: config } = jsonParseNoThrow(node_fs_1.default.readFileSync(file, 'utf8'));
        log.debug('Found package.json', file, 'and config:', config);
        if (!config)
            return {};
        (0, condition_1.assertIsObject)(config, `Bad "importSorter" config in "${file}"`);
        return config;
    });
}
function jsonParseNoThrow(content) {
    try {
        return JSON.parse(content);
    }
    catch (e) {
        const log = (0, common_1.logger)('format-imports.jsonParseNoThrow');
        const msg = e instanceof Error ? e.message : `${e}`;
        log.warn('Failed to parse JSON content and found error:', msg);
        return {};
    }
}
function readConfigTilRoot(fileNames, readConfig) {
    let config = {};
    for (const file of fileNames) {
        const c = readConfig(file);
        config = (0, merge_1.mergeConfig)(c, config);
        if (config.root)
            break;
    }
    return config;
}
function enhanceEol(config, detectEol) {
    const log = (0, common_1.logger)('format-imports.enhanceEol');
    log.debug('Determining EOL');
    if (config.eol)
        return config;
    const nl = detectEol();
    const eol = nl === '\r' ? 'CR' : nl === '\r\n' ? 'CRLF' : nl === '\n\r' ? 'LFCR' : 'LF';
    return (0, merge_1.mergeConfig)({ eol }, config);
}
exports.enhanceEol = enhanceEol;
/**
 * Load config from given file, e.g. _path/to/import-sorter.json_.
 *
 * Will throw an error if file is unreadable or content is not a valid JSON object.
 */
function loadConfigFromJsonFile(fileName) {
    if (!fileName)
        return {};
    const config = JSON.parse(node_fs_1.default.readFileSync(fileName, 'utf8'));
    (0, condition_1.assertIsObject)(config, `Bad config in "${fileName}"`);
    return config;
}
exports.loadConfigFromJsonFile = loadConfigFromJsonFile;
//# sourceMappingURL=importSorter.js.map