"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeConfig = exports.DEFAULT_MERGER = void 0;
const merge_options_1 = require("@dozerg/merge-options");
/**
 * Default merge rules for [mergeConfig](#mergeConfig), which are:
 *
 * - {@link exclude}, {@link excludeGlob} and {@link keepUnused} arrays will be concatenated instead of replaced;
 * - {@link sortRules} object will be merged instead of replaced;
 * - All other fields will be replaced and the latter config takes precedence.
 *
 * When creating your own merge policy, make sure to inherit the default merger and just override
 * the ones different.
 */
exports.DEFAULT_MERGER = {
    exclude: (0, merge_options_1.concatArray)(),
    excludeGlob: (0, merge_options_1.concatArray)(),
    keepUnused: (0, merge_options_1.concatArray)(),
    sortRules: (0, merge_options_1.customize)((a, b) => {
        const aa = a === 'none' ? { paths: 'none', names: 'none' } : a;
        const bb = b === 'none' ? { paths: 'none', names: 'none' } : b;
        return { ...aa, ...bb };
    }),
    ignoreESLintRules: (0, merge_options_1.concatArrayEx)(),
};
/**
 * Merge multiple configs together. The latter takes precedence if values have conflicts.
 *
 * This function is preferred to `{...config1, ...config2}` because some keys need to be
 * merged instead of overwritten, e.g. `exclude`. Please refer to  [DEFAULT_MERGER](#DEFAULT_MERGER)
 * for more details.
 *
 * Example:
 * ```ts
 * const config1 = { maxLineLength: 80, tabSize: 2 };
 * const config2 = { maxLineLength: 100 };
 *
 * const config = mergeConfig(config1, config2);  // { maxLineLength: 100, tabSize: 2 }
 * ```
 *
 * @param configs - An array of config objects
 */
function mergeConfig(...configs) {
    return (0, merge_options_1.mergeOptions)(exports.DEFAULT_MERGER, ...configs);
}
exports.mergeConfig = mergeConfig;
//# sourceMappingURL=merge.js.map