"use strict";
/* eslint-disable @typescript-eslint/no-unsafe-return */
/* eslint-disable @typescript-eslint/ban-types */
Object.defineProperty(exports, "__esModule", { value: true });
exports.apply = exports.extractOptions = void 0;
const condition_1 = require("@dozerg/condition");
const common_1 = require("../../../common");
/**
 * Extract options for ESLint rules.
 * All rules should have the same options structure, e.g. indent and \@typescript-eslint/indent
 * @returns The resolved options of the rules specified by _keys_.
 */
function extractOptions(config, rules, defaultOptions, ...keys) {
    const log = (0, common_1.logger)(`format-imports.extractOptions`);
    const results = keys
        .map(key => {
        const o = extractRuleOptions(rules, key, defaultOptions);
        if (o === undefined)
            return undefined;
        const ignored = isRuleIgnored(config, key);
        log.info(ignored ? 'Ignored' : 'Found', `ESLint rule ${key}:`, o);
        return ignored ? undefined : o;
    })
        .filter(condition_1.isNonNull);
    // If multiple opitons are found, the last one takes precedence.
    return results.length < 1 ? undefined : results[results.length - 1];
}
exports.extractOptions = extractOptions;
function extractRuleOptions(rules, key, defaultOptions) {
    const rule = rules[key];
    if (rule === undefined || rule === 0 || rule === 'off')
        return undefined;
    if (Array.isArray(rule)) {
        const level = rule[0];
        if (level === 0 || level === 'off')
            return undefined;
        const opt = rule[1];
        return opt === undefined
            ? defaultOptions
            : (0, condition_1.isObject)(opt) && (0, condition_1.isObject)(defaultOptions)
                ? { ...defaultOptions, ...opt }
                : opt;
    }
    return defaultOptions;
}
function isRuleIgnored({ ignoreESLintRules: patterns }, key) {
    if (!patterns)
        return false;
    for (const p of typeof patterns === 'string' ? [patterns] : patterns) {
        const r = new RegExp(p);
        if (r.test(key))
            return true;
    }
    return false;
}
/**
 * Apply a number of `translators` to `config`.
 *
 * @param config - The original configuration
 * @param rules - ESLint rules
 * @param translators - An array of translations.
 * @returns The updated config and processed data if there are any
 */
function apply(config, rules, ...translators) {
    const [translator, ...rest] = translators;
    if (!translator)
        return { config };
    const { config: c, processed } = applyOne(config, rules, translator);
    return applyNext(c, rules, processed, ...rest);
}
exports.apply = apply;
function applyOne(config, rules, translator, processed) {
    const { config: c, processed: p } = translator(config, rules, translator.name);
    return { config: c, processed: merge(processed, p) };
}
function applyNext(config, rules, processed, ...translations) {
    const [translator, ...rest] = translations;
    if (!translator)
        return { config, processed };
    const { config: c, processed: p } = applyOne(config, rules, translator, processed);
    return applyNext(c, rules, merge(processed, p), ...rest);
}
function merge(t, u) {
    return t ? (u ? { ...t, ...u } : t) : u;
}
//# sourceMappingURL=helper.js.map