"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatSourceWithoutFile = exports.formatSourceFromFile = exports.SUPPORTED_EXTENSIONS = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const tmp_1 = __importDefault(require("tmp"));
const common_1 = require("../../common");
const config_1 = require("../config");
const format_1 = require("./format");
const vue_1 = require("./vue");
/**
 * File extensions supported.
 */
exports.SUPPORTED_EXTENSIONS = ['js', 'ts', 'jsx', 'tsx', 'vue'];
/**
 * Format given source text from a file, asynchronously.
 *
 * This function will try to find _tsconfig.json_ and ESLint config relating to the source file,
 * and merge them to the base config provided.
 *
 * `options` can be used to change _tsconfig.json_ and ESLint config loading behavior for testing
 * purpose.
 *
 * @see {@link formatSourceFromFile.sync}
 *
 * @param text - Source text
 * @param fileName - Source file name
 * @param config - Base config
 * @param options - Internal/testing options
 *
 * @returns Promise of the result text or `undefined` if nothing changes.
 */
async function formatSourceFromFile(text, fileName, config, options) {
    const log = (0, common_1.logger)('format-imports.formatSourceFromFile');
    log.debug('Config:', config);
    log.debug('Options:', options);
    const allConfig = await (0, config_1.enhanceConfig)(config, fileName, options);
    if (fileName.endsWith('.vue'))
        return (0, vue_1.formatVueSource)(text, fileName, allConfig);
    return (0, format_1.formatSource)(text, fileName, allConfig);
}
exports.formatSourceFromFile = formatSourceFromFile;
/**
 * Format given source text from a file, synchronoussly.
 *
 * This function will try to find _tsconfig.json_ relating to the source file,
 * and merge them to the base config provided.
 *
 * ESLint config will NOT be read because it's supported only by {@link formatSourceFromFile}.
 *
 * `options` can be used to change _tsconfig.json_ loading behavior for testing
 * purpose.
 *
 * @see {@link formatSourceFromFile}
 *
 * @param text - Source text
 * @param fileName - Source file name
 * @param config - Base config
 * @param options - Internal/testing options
 *
 * @returns The result text or `undefined` if nothing changes.
 */
formatSourceFromFile.sync = (text, fileName, config, options) => {
    const log = (0, common_1.logger)('format-imports.formatSourceFromFile.sync');
    log.debug('Config:', config);
    log.debug('Options:', options);
    const allConfig = config_1.enhanceConfig.sync(config, fileName, options);
    return (0, format_1.formatSource)(text, fileName, allConfig);
};
/**
 * Format given source text without knowing the source file, asynchronously.
 *
 * This function will try to find _tsconfig.json_ and ESLint config relating to the source file,
 * and merge them to the base config provided.
 *
 * `options` can be used to change _tsconfig.json_ and ESLint config loading behavior for testing
 * purpose.
 *
 * @see {@link formatSourceWithoutFile.sync}
 *
 * @param text - Source text
 * @param extension - File extension to reveal the source language
 * @param config - Base config
 * @param options - Internal/testing options
 *
 * @returns Promise of the result text or `undefined` if nothing changes.
 */
async function formatSourceWithoutFile(text, extension, config, options) {
    tmp_1.default.setGracefulCleanup();
    const { fd, name } = tmp_1.default.fileSync({ prefix: 'format-imports-lib', postfix: `.${extension}` });
    node_fs_1.default.writeSync(fd, text);
    const skipTsConfig = options?.skipTsConfig || !options?.tsConfigPath;
    const skipEslintConfig = options?.skipEslintConfig || !options?.eslintConfigPath;
    const newOpt = options && { ...options, skipTsConfig, skipEslintConfig };
    return formatSourceFromFile(text, name, config, newOpt);
}
exports.formatSourceWithoutFile = formatSourceWithoutFile;
/**
 * Format given source text without knowing the source file, synchronously.
 *
 * This function will try to find _tsconfig.json_ relating to the source file,
 * and merge them to the base config provided.
 *
 * ESLint config will NOT be read because it's supported only by {@link formatSourceWithoutFile}.
 *
 * `options` can be used to change _tsconfig.json_ loading behavior for testing
 * purpose.
 *
 * @see {@link formatSourceWithoutFile}
 *
 * @param text - Source text
 * @param extension - File extension to reveal the source language
 * @param config - Base config
 * @param options - Internal/testing options
 *
 * @returns The result text or `undefined` if nothing changes.
 */
formatSourceWithoutFile.sync = (text, extension, config, options) => {
    tmp_1.default.setGracefulCleanup();
    const { fd, name } = tmp_1.default.fileSync({ prefix: 'format-imports-lib', postfix: `.${extension}` });
    node_fs_1.default.writeSync(fd, text);
    const skipTsConfig = options?.skipTsConfig || !options?.tsConfigPath;
    const newOpt = options && { ...options, skipTsConfig };
    return formatSourceFromFile.sync(text, name, config, newOpt);
};
//# sourceMappingURL=index.js.map