"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const immutable_1 = require("immutable");
const is_builtin_module_1 = __importDefault(require("is-builtin-module"));
const typescript_1 = require("typescript");
const condition_1 = require("@dozerg/condition");
const compose_1 = require("../compose");
const helper_1 = require("./helper");
const path_1 = require("./path");
const Statement_1 = __importDefault(require("./Statement"));
class ImportNode extends Statement_1.default {
    static fromDecl(node, args) {
        const { importClause, moduleSpecifier, assertClause } = node;
        if (!moduleSpecifier || moduleSpecifier.kind !== typescript_1.SyntaxKind.StringLiteral)
            return undefined;
        const moduleIdentifier = moduleSpecifier.text;
        if (!moduleIdentifier.trim())
            return undefined;
        const { defaultName, binding, isScript, isTypeOnly } = getDefaultAndBinding(importClause);
        const assertEntries = getAssertEntries(assertClause);
        return new ImportNode(node, moduleIdentifier, args, defaultName, binding, isScript, isTypeOnly, assertEntries);
    }
    static fromEqDecl(node, args) {
        const { moduleReference } = node;
        if (!moduleReference || moduleReference.kind !== typescript_1.SyntaxKind.ExternalModuleReference)
            return undefined;
        const { expression } = moduleReference;
        if (!expression || expression.kind !== typescript_1.SyntaxKind.StringLiteral)
            return undefined;
        const moduleIdentifier = expression.text;
        const defaultName = node.name.text;
        return new ImportNode(node, moduleIdentifier, args, defaultName);
    }
    constructor(node, moduleIdentifier, args, defaultName, binding, isScript = false, isTypeOnly = false, assertEntries) {
        super(args);
        this.node_ = node;
        this.moduleIdentifier = (0, path_1.normalizePath)(moduleIdentifier, args.config);
        this.defaultName_ = defaultName;
        this.binding_ = binding;
        this.isScript = isScript;
        this.isTypeOnly = isTypeOnly;
        this.assertEntries = assertEntries;
    }
    get defaultName() {
        return this.defaultName_;
    }
    get binding() {
        return this.binding_;
    }
    get flagType() {
        if (this.isScript)
            return 'scripts';
        if (this.defaultName_)
            return this.binding_ ? 'multiple' : 'single';
        if (!this.binding_)
            return 'multiple';
        if (this.binding_.type === 'namespace')
            return 'namespace';
        return this.binding_.names.length === 1 ? 'single' : 'multiple';
    }
    get isBuiltIn() {
        return (0, is_builtin_module_1.default)(this.moduleIdentifier);
    }
    matchPath(regex) {
        if (regex.test(this.moduleIdentifier))
            return true;
        if (this.isBuiltIn) {
            const realId = (0, path_1.removeNodeProtocol)(this.moduleIdentifier);
            return regex.test(realId);
        }
        return false;
    }
    allNames() {
        const r = [];
        if (this.defaultName_)
            r.push(this.defaultName_);
        if (this.binding_?.type === 'namespace')
            r.push(this.binding_.alias);
        else if (this.binding_?.type === 'named') {
            const names = this.binding_.names.map(n => n.aliasName ?? n.propertyName);
            r.push(...names);
        }
        return r;
    }
    empty() {
        return !this.isScript && !this.defaultName_ && !this.binding_;
    }
    removeUnusedNames(usage, keepUnused) {
        const { unusedNodes } = usage;
        if (this.isScript)
            return;
        const keepUnusedName = keepUnused?.forPath(this.moduleIdentifier);
        if (unusedNodes && unusedNodes.includes(this) && !keepUnusedName) {
            this.defaultName_ = undefined;
            this.binding_ = undefined;
            return;
        }
        if (!isNameUsed(this.defaultName_, usage, keepUnusedName))
            this.defaultName_ = undefined;
        if (this.binding_) {
            if (this.binding_.type === 'named') {
                this.binding_.names = this.binding_.names.filter(n => isNameUsed(n, usage, keepUnusedName));
                if (!this.binding_.names.length)
                    this.binding_ = undefined;
            }
            else if (!isNameUsed(this.binding_.alias, usage, keepUnusedName)) {
                this.binding_ = undefined;
            }
        }
    }
    /**
     * @returns true if `node` is fully merged to `this`;
     *          false if `node` still has names or comments thus can't be ignored.
     */
    merge(node) {
        const { isTypeOnly } = this;
        if (this.moduleIdentifier !== node.moduleIdentifier ||
            this.node_.kind !== node.node_.kind ||
            this.isScript !== node.isScript ||
            isTypeOnly !== node.isTypeOnly ||
            !this.canMergeComments(node) ||
            0 !== compareAssertEntries(this.assertEntries, node.assertEntries))
            return false;
        this.removeBindingDefault(node.defaultName_);
        node.removeBindingDefault(this.defaultName_);
        const r1 = this.mergeBinding(node, !isTypeOnly);
        const r2 = this.mergeDefaultName(node, !isTypeOnly);
        return r1 && r2 && this.mergeComments(node);
    }
    checkBindingDefault() {
        // `import A, {default as A} from 'x'` => `import A from 'x'`
        if (this.defaultName_)
            this.removeBindingDefault(this.defaultName_);
        // `import {default as A, B} from 'x'` => `import A, {B} from 'x'`
        // `import type {default as A} from 'x'` => `import type A from 'x'`
        else
            this.defaultName_ = this.pickBindingDefault(this.isTypeOnly);
    }
    compose(config) {
        const { leadingText, trailingText, tailingLength } = this.composeComments(config);
        const importText = this.composeImport(tailingLength, config);
        return leadingText + importText + trailingText;
    }
    removeBindingDefault(defaultName) {
        if (!defaultName || this.binding_?.type !== 'named')
            return;
        this.binding_.names = this.binding_.names.filter(a => a.aliasName !== defaultName || a.propertyName !== 'default');
        if (this.binding_.names.length < 1)
            this.binding_ = undefined;
    }
    /**
     * Find and remove 'default as X' from binding names and returns 'X'.
     *
     * If whenSingle is true and names array length is not 1, do nothing.
     */
    pickBindingDefault(whenSingle = false) {
        if (this.binding_?.type !== 'named')
            return undefined;
        if (whenSingle && this.binding_.names.length > 1)
            return undefined;
        const name = this.binding_.names.find(a => a.propertyName === 'default')?.aliasName;
        this.removeBindingDefault(name);
        return name;
    }
    mergeBinding(node, takeOver) {
        const { binding_: b1 } = this;
        const { binding_: b2 } = node;
        if (!b2)
            return true;
        else if (!b1) {
            if (!takeOver)
                return false;
            this.binding_ = b2;
            node.binding_ = undefined;
            return true;
        }
        else if (b1.type === 'namespace') {
            if (b2.type === 'namespace' && b1.alias === b2.alias) {
                node.binding_ = undefined;
                return true;
            }
            return false;
        }
        else if (b2.type === 'namespace')
            return false;
        b1.names.push(...b2.names);
        node.binding_ = undefined;
        return true;
    }
    mergeDefaultName(node, takeOver) {
        if (this.defaultName_ && node.defaultName_ && this.defaultName_ !== node.defaultName_)
            return false;
        if (!this.defaultName_) {
            if (!takeOver)
                return !node.defaultName_;
            this.defaultName_ = node.defaultName_;
        }
        node.defaultName_ = undefined;
        return true;
    }
    composeImport(commentLength, config) {
        switch (this.node_.kind) {
            case typescript_1.SyntaxKind.ImportDeclaration:
                return this.composeDecl(commentLength, config);
            case typescript_1.SyntaxKind.ImportEqualsDeclaration:
                return this.composeEqDecl(commentLength, config);
        }
    }
    // import A = require('B');
    composeEqDecl(extraLength, config) {
        const { quote, wrap } = config;
        const path = this.moduleIdentifier;
        const name = this.defaultName_;
        (0, condition_1.assertNonNull)(name);
        return (0, compose_1.composeParts)([
            (0, compose_1.StringPart)(`import ${name} =`, true),
            (0, compose_1.StringPart)(`require(${quote(path)})`, !wrap.parts),
            (0, compose_1.SemiPart)(extraLength),
        ], config);
    }
    /**
     * Script import:
     * ```
     *    import 'A';
     * ```
     *
     * Default import:
     * ```
     *    import A from 'B';
     * ```
     *
     * Binding names import:
     * ```
     *    import { A, B as C } from 'D';
     *    import {
     *      A, B,
     *      C as D,
     *    } from 'E';
     * ```
     *
     * Namespace import:
     * ```
     *    import * as A from 'B';
     * ```
     *
     * Mixed examples:
     * ```
     *    import A, { B, C } from 'D';
     *    import A, {
     *      B,
     *      C as D,
     *      E,
     *    } from 'F';
     *    import A, * as B from 'C';
     *    import A, { default as B, C, D } from 'E';
     * ```
     *
     * With assert clause
     * ```
     *    import A from 'a' assert { type: 'json' };
     * ```
     */
    composeDecl(extraLength, config) {
        const { quote, semi, wrap } = config;
        const path = this.moduleIdentifier;
        const ending = quote(path);
        if (this.isScript)
            return `import ${ending}${semi}`;
        const verb = 'import' + (this.isTypeOnly ? ' type' : '');
        const from = `from ${ending}`;
        const noWrap = !wrap.parts;
        if (!this.binding_) {
            // import A from 'a' assert { x: 'y' }
            return (0, compose_1.composeParts)([
                (0, compose_1.StringPart)(`${verb} ${this.defaultName_}`, true),
                (0, compose_1.StringPart)(from, noWrap),
                (0, compose_1.AssertPart)(this.assertEntries, noWrap),
                (0, compose_1.SemiPart)(extraLength),
            ], config);
        }
        else if (this.binding_.type === 'namespace') {
            // import * as A from 'a' assert { x: 'y' }, or
            // import A, * as B from 'a' assert { x: 'y' }
            const alias = `* as ${this.binding_.alias}`;
            return (0, compose_1.composeParts)([
                ...(this.defaultName_
                    ? [(0, compose_1.StringPart)(`${verb} ${this.defaultName_},`, true), (0, compose_1.StringPart)(alias, noWrap)]
                    : [(0, compose_1.StringPart)(`${verb} ${alias}`, true)]),
                (0, compose_1.StringPart)(from, noWrap),
                (0, compose_1.AssertPart)(this.assertEntries, noWrap),
                (0, compose_1.SemiPart)(extraLength),
            ], config);
        }
        // import { A, B } from 'a' assert { x: 'y' }, or
        // import A, { B, C } from 'a' assert { x: 'y' }
        const { withDefault, withoutDefault } = wrap;
        const maxWords = this.defaultName_ ? withDefault - 1 : withoutDefault;
        return (0, compose_1.composeParts)([
            (0, compose_1.StringPart)(this.defaultName_ ? `${verb} ${this.defaultName_},` : verb, true),
            (0, compose_1.NamedPart)(this.binding_.names, from, maxWords, !this.defaultName_ && noWrap),
            (0, compose_1.AssertPart)(this.assertEntries, noWrap),
            (0, compose_1.SemiPart)(extraLength),
        ], config);
    }
}
exports.default = ImportNode;
function isNameUsed(name, usage, keepUnused) {
    if (!name)
        return false;
    const { unusedNames, usedNames } = usage;
    const n = typeof name === 'string' ? name : name.aliasName ?? name.propertyName;
    if (!n)
        return false;
    if (keepUnused?.(n))
        return true;
    // `unusedNames` (from TS compiler) gives more accurate results
    // than `usedNames` (from manual parsing).
    if (unusedNames)
        return !unusedNames.has(n);
    if (usedNames)
        return usedNames.has(n);
    return true; // Keep it for safety
}
function getDefaultAndBinding(importClause) {
    if (!importClause)
        return { isScript: true };
    const { name, namedBindings, isTypeOnly } = importClause;
    const defaultName = name?.text;
    const binding = getBinding(namedBindings);
    return { defaultName, binding, isScript: false, isTypeOnly };
}
function getAssertEntries(assertClause) {
    if (!assertClause)
        return undefined;
    const { kind, elements } = assertClause;
    if (kind !== typescript_1.SyntaxKind.AssertClause)
        return undefined;
    const entries = (0, immutable_1.Map)(elements.map(({ name, value }) => [name.text, value.text]));
    return entries.isEmpty() ? undefined : entries;
}
function compareAssertEntries(assert1, assert2) {
    if (assert1 === assert2)
        return 0;
    else if (assert1 === undefined)
        return -1;
    else if (assert2 === undefined)
        return 1;
    const it1 = assert1.sort().entries();
    const it2 = assert2.sort().entries();
    let v1 = it1.next();
    let v2 = it2.next();
    for (; !v1.done && !v2.done; v1 = it1.next(), v2 = it2.next()) {
        const r = compareKeyValue(v1.value, v2.value);
        if (r !== 0)
            return r;
    }
    return v1.done === v2.done ? 0 : v1.done ? -1 : 1;
}
function compareKeyValue([k1, v1], [k2, v2]) {
    const r = k1.localeCompare(k2);
    if (r !== 0)
        return r;
    return v1.localeCompare(v2);
}
function getBinding(nb) {
    if (!nb)
        return undefined;
    if (nb.kind === typescript_1.SyntaxKind.NamespaceImport)
        return { type: 'namespace', alias: nb.name.text };
    const names = nb.elements.map(helper_1.getNameBinding);
    return names.length < 1 ? undefined : { type: 'named', names };
}
//# sourceMappingURL=ImportNode.js.map