"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isDisabled = exports.parseLineRanges = void 0;
const typescript_1 = __importDefault(require("typescript"));
function parseLineRanges(node, p) {
    const { sourceFile, sourceText } = p;
    const declEnd = node.getEnd();
    const declLineRange = transformRange({ pos: node.getStart(sourceFile), end: declEnd }, sourceFile);
    const { fileComments, fullStart, leadingNewLines, commentsStart, leadingComments } = parseLeadingComments(node, declLineRange, p);
    const trailingComments = typescript_1.default
        .getTrailingCommentRanges(sourceText, declEnd)
        ?.map(transformComment.bind(undefined, sourceFile, sourceText));
    // All tailing comments text should keep unchanged including the leading spaces
    const commentsEnd = trailingComments?.slice(-1)[0]?.end.pos ?? declEnd;
    const trailingCommentsText = sourceText.slice(declEnd, commentsEnd);
    const declAndCommentsLineRange = transformRange({ pos: commentsStart, end: commentsEnd }, sourceFile);
    p.prevCommentEnd = declAndCommentsLineRange.end; // Save comment end pos for the next statement.
    const { trailingNewLines, fullEnd: fullEndPos, eof, } = getTrailingNewLines(sourceText, commentsEnd);
    return {
        fileComments,
        fullStart,
        leadingNewLines,
        leadingComments,
        declLineRange,
        trailingComments,
        trailingCommentsText,
        declAndCommentsLineRange,
        trailingNewLines,
        fullEnd: transformPos(fullEndPos, sourceFile),
        eof,
    };
}
exports.parseLineRanges = parseLineRanges;
function isDisabled(comments) {
    for (const c of comments ?? []) {
        // ts-import-sorter: disable
        /* ts-import-sorter: disable */
        if (/^\s*\/\/\s*ts-import-sorter\s*:\s*disable\s*$/.test(c.text) ||
            /^\s*\/\*\s*ts-import-sorter\s*:\s*disable\s*\*\/$/.test(c.text))
            return true;
    }
    return false;
}
exports.isDisabled = isDisabled;
function transformRange({ pos, end }, sourceFile) {
    return {
        start: transformPos(pos, sourceFile),
        end: transformPos(end, sourceFile),
    };
}
function transformPos(pos, sourceFile) {
    return { pos, ...sourceFile.getLineAndCharacterOfPosition(pos) };
}
function parseLeadingComments(node, declLineRange, p) {
    const { sourceFile, sourceText, prevCommentEnd, checkFileComments } = p;
    const fullStart = prevCommentEnd ?? transformPos(node.getFullStart(), sourceFile);
    const comments = typescript_1.default
        .getLeadingCommentRanges(sourceText, fullStart.pos)
        ?.map(transformComment.bind(undefined, sourceFile, sourceText));
    if (comments && comments.length > 0) {
        // Skip global comments that separated by empty line(s) or triple-slash comment(s)
        const results = [];
        let nextStartLine = declLineRange.start.line;
        for (let i = comments.length - 1; i >= 0; --i) {
            const comment = comments[i];
            const { start, end, text } = comment;
            const leadingNewLines = nextStartLine - end.line;
            comment.trailingNewLines = leadingNewLines;
            if (checkFileComments && (1 < leadingNewLines || isTripleSlashComment(text))) {
                const leadingComments = results.reverse();
                const commentsStart = results[0]?.start.pos ?? declLineRange.start.pos;
                const fileComments = comments.slice(0, i + 1);
                return {
                    fullStart: transformPos(end.pos, sourceFile),
                    leadingNewLines,
                    commentsStart,
                    leadingComments,
                    fileComments,
                };
            }
            results.push(comment);
            nextStartLine = start.line;
        }
    }
    // Find out leading empty lines
    const commentsStart = comments?.[0]?.start.pos ?? declLineRange.start.pos;
    const leadingNewLines = sourceText.slice(fullStart.pos, commentsStart).match(/\n/g)?.length ?? 0;
    return { fullStart, leadingNewLines, commentsStart, leadingComments: comments };
}
function transformComment(sourceFile, sourceText, range) {
    const text = sourceText.slice(range.pos, range.end);
    return { ...range, ...transformRange(range, sourceFile), text };
}
function isTripleSlashComment(text) {
    return text.trimLeft().search(/^\/\/\/\s*</) >= 0;
}
function getTrailingNewLines(sourceText, pos) {
    const input = sourceText.slice(pos);
    const text = /^\s*/u.exec(input)?.[0];
    const trailingNewLines = text?.match(/\n/g)?.length ?? 0;
    return { trailingNewLines, fullEnd: pos + (text?.length ?? 0), eof: input === text };
}
//# sourceMappingURL=lines.js.map