"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseSource = void 0;
const typescript_1 = require("typescript");
const common_1 = require("../../common");
const ExportNode_1 = __importDefault(require("./ExportNode"));
const ImportNode_1 = __importDefault(require("./ImportNode"));
const lines_1 = require("./lines");
const ParseParams_1 = __importDefault(require("./ParseParams"));
function parseSource(sourceFile, sourceText, config, options) {
    const p = new ParseParams_1.default(sourceFile, sourceText);
    for (const node of sourceFile.statements)
        if (!process(node, p, config, options))
            break;
    return p;
}
exports.parseSource = parseSource;
/**
 * @returns true to continue processing the next node; Or false to stop processing the whole source
 */
function process(node, p, config, options) {
    const { force, formatExports } = config;
    const { fileComments, fullStart, leadingNewLines, leadingComments, 
    // declLineRange,
    trailingComments, trailingCommentsText, declAndCommentsLineRange, trailingNewLines, fullEnd, eof, } = (0, lines_1.parseLineRanges)(node, p);
    if (!force && (0, lines_1.isDisabled)(fileComments)) {
        (0, common_1.logger)('parser.process').info('Disable comment found. Ignoring file.');
        return false;
    }
    if (isUseDirective(node))
        return true; // Skip 'use' directives
    p.checkFileComments = false; // No more checks for global comments after non-'use strict' statement
    const range = {
        ...declAndCommentsLineRange,
        fullStart,
        leadingNewLines,
        trailingNewLines,
        fullEnd,
        eof,
    };
    const disabled = (0, lines_1.isDisabled)(leadingComments) || (0, lines_1.isDisabled)(trailingComments);
    const a = { range, leadingComments, trailingCommentsText, config };
    if (node.kind === typescript_1.SyntaxKind.ImportDeclaration)
        handleImport(p, disabled, range, () => ImportNode_1.default.fromDecl(node, a));
    else if (node.kind === typescript_1.SyntaxKind.ImportEqualsDeclaration)
        handleImport(p, disabled, range, () => ImportNode_1.default.fromEqDecl(node, a));
    else
        handleOthers(p, formatExports === false || disabled, range, node, a, options);
    return true;
}
function handleImport(p, disabled, range, importNode) {
    if (!disabled) {
        p.updateImportInsertPoint(range);
        if (p.addImport(importNode()))
            return;
    }
    p.addUnhandledImportOrExport();
}
function handleOthers(p, disabled, range, node, a, options) {
    parseId(node, p, options);
    p.updateImportInsertPoint(range);
    if (node.kind === typescript_1.SyntaxKind.ExportDeclaration) {
        if (!disabled) {
            const n = ExportNode_1.default.fromDecl(node, a);
            if (p.addExport(n))
                return;
        }
        p.addUnhandledImportOrExport();
    }
    else if (node.kind === typescript_1.SyntaxKind.ExportAssignment)
        p.addUnhandledImportOrExport();
}
/**
 * Traverse node and find out all referenced names.
 * The result is used only in removing unused names.
 *
 * This function is deprecated because it's less accurate and reliable
 * compared to TS compiler error/warning messages.
 *
 * Keep the code just for regression purposes.
 * @deprecated In favor to TS compiler error/warning messages.
 */
function parseId(node, p, options) {
    const { sourceFile, allIds } = p;
    switch (node.kind) {
        case typescript_1.SyntaxKind.Identifier:
            allIds.add(node.getText(sourceFile));
            break;
        case typescript_1.SyntaxKind.JsxElement:
        case typescript_1.SyntaxKind.JsxSelfClosingElement:
        case typescript_1.SyntaxKind.JsxFragment:
            // See: https://github.com/ionic-team/stencil/blob/master/BREAKING_CHANGES.md#import--h--is-required
            allIds.add(options?.jsxFactory === 'h' ? 'h' : 'React');
            break;
    }
    node.forEachChild(n => parseId(n, p, options));
}
/** Is a node a "use" directive such as "use strict" or "use client" */
function isUseDirective(node) {
    if (node.kind !== typescript_1.SyntaxKind.ExpressionStatement)
        return false;
    const { expression } = node;
    if (!expression || expression.kind !== typescript_1.SyntaxKind.StringLiteral)
        return false;
    return /^use (strict|client)$/.test(expression.text);
}
//# sourceMappingURL=parser.js.map