"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUnusedIds = void 0;
const typescript_1 = __importDefault(require("typescript"));
const condition_1 = require("@dozerg/condition");
const common_1 = require("../../common");
const path_1 = require("./path");
var UnusedCode;
(function (UnusedCode) {
    UnusedCode[UnusedCode["SINGLE_1"] = 6133] = "SINGLE_1";
    UnusedCode[UnusedCode["SINGLE_2"] = 6196] = "SINGLE_2";
    UnusedCode[UnusedCode["ALL"] = 6192] = "ALL";
})(UnusedCode || (UnusedCode = {}));
function getUnusedIds(usedNames, importNodes, fileName, sourceFile, tsCompilerOptions) {
    const UNUSED_CODE = new Set(Object.values(UnusedCode).filter(condition_1.isNumber));
    const unusedNames = new Set();
    const unusedNodes = Array();
    const options = prepareOptions(tsCompilerOptions);
    const host = mockHost(fileName, sourceFile, options);
    const program = typescript_1.default.createProgram([fileName], options, host);
    try {
        // https://github.com/microsoft/TypeScript/wiki/API-Breaking-Changes#program-interface-changes
        program
            .getSemanticDiagnostics(sourceFile)
            .filter(m => m.file === sourceFile && UNUSED_CODE.has(m.code))
            .forEach(m => transform(m, importNodes, unusedNames, unusedNodes));
        return { usedNames, unusedNames, unusedNodes };
    }
    catch (e) {
        (0, common_1.logger)('parser.getUnusedIds').error('getSemanticDiagnostics failed:', e);
        return { usedNames };
    }
}
exports.getUnusedIds = getUnusedIds;
function prepareOptions(options) {
    return {
        ...(options ?? {}),
        noEmit: true,
        noUnusedLocals: true,
        allowJs: true,
        checkJs: true,
    };
}
function mockHost(fileName, sourceFile, options) {
    const host = typescript_1.default.createCompilerHost(options);
    // Normalize the path before comparing because it can be both '/' and '\' in Windows
    const fn = (0, path_1.normalizePath)(fileName);
    return {
        ...host,
        getSourceFile: f => ((0, path_1.normalizePath)(f) === fn ? sourceFile : undefined),
        // The following costs significant performance:
        // getSourceFile: (f, l, e, c) => {
        //   logger('parser.mockHost').debug('fileName:', f);
        //   return normalizePath(f) === fn ? sourceFile : host.getSourceFile(f, l, e, c);
        // },
    };
}
function transform(m, importNodes, unusedNames, unusedNodes) {
    const { code, start: pos, messageText: text } = m;
    const log = (0, common_1.logger)('parser.transform');
    if (pos === undefined || typeof text !== 'string')
        return;
    if (code === UnusedCode.SINGLE_1 || code === UnusedCode.SINGLE_2) {
        // ts(6133): 'XXX' is declared but its value is never read.
        // ts(6196): 'XXX' is declared but never used.
        // `[^']+` is used instead of `\w+` for unicode characters.
        const id = /^'([^']+)' is declared but/.exec(text)?.[1];
        if (!id) {
            log.warn(`Cannot parse identifier for error code=${code} from message='${text}'`);
            return;
        }
        unusedNames.add(id);
    }
    else if (code === UnusedCode.ALL) {
        // ts(6192): All imports in import declaration are unused.
        const node = importNodes.find(n => n.withinDeclRange(pos));
        if (!node) {
            log.warn(`Cannot find node at pos=${pos} for error code=${code}`);
            return;
        }
        node.allNames().forEach(n => unusedNames.add(n));
        unusedNodes.push(node);
    }
}
//# sourceMappingURL=unused.js.map