"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const is_builtin_module_1 = __importDefault(require("is-builtin-module"));
const config_1 = require("../../config");
const compare_1 = require("./compare");
const merge_1 = require("./merge");
class SortGroup {
    constructor({ flags: originFlags, regex, importType, builtin, sortImportsBy, sort, subGroups }, parent, eslint) {
        this.nodes_ = []; // Fall-back group for non-script imports
        this.scripts_ = []; // Fall-back group for script imports
        this.ignoreSubGroups_ = false;
        const flags = breakFlags(originFlags);
        const flags1 = SortGroup.inferFlags1(flags, parent.flags);
        this.regex_ = regex || regex === '' ? RegExp(regex) : undefined;
        this.importType_ = importType;
        this.builtin_ = builtin;
        if (eslint?.ignoreSorting) {
            this.sortImportsBy_ = parent.sortImportsBy ?? 'paths';
            this.sorter_ = parent.sorter;
        }
        else {
            this.sortImportsBy_ = sortImportsBy ?? parent.sortImportsBy ?? 'paths';
            this.sorter_ = parent.sorter
                ? (0, compare_1.updateSorterWithRules)(parent.sorter, sort)
                : (0, compare_1.sorterFromRules)(sort);
        }
        this.subGroups_ = this.calcSubGroups(subGroups, flags1, eslint);
        this.flags_ = SortGroup.inferFlags2(flags, flags1, this.subGroups_);
        this.eslintGroupOrder_ = eslint?.groupOrder;
        this.aliasFirst_ = !!eslint?.aliasFirst;
    }
    /**
     * @param node - The node to be added
     * @param fallBack - Whether to add node if this is a fall-back group
     * @returns Whether node is added to this group
     */
    add(node, fallBack = false) {
        const { flagType, moduleIdentifier, isTypeOnly } = node;
        const isBuiltIn = (0, is_builtin_module_1.default)(moduleIdentifier);
        if ((this.importType_ ?? isTypeOnly) !== isTypeOnly)
            return false;
        if ((this.builtin_ ?? isBuiltIn) !== isBuiltIn)
            return false;
        if (!isFlagIncluded(flagType, this.flags_))
            return false;
        const isScript = flagType === 'scripts';
        if (this.regex_) {
            if (!node.matchPath(this.regex_))
                return false;
            if (this.addToSubGroup(node, fallBack))
                return true;
            if (!fallBack && this.addToSubGroup(node, true))
                return true;
            isScript ? this.scripts_.push(node) : this.nodes_.push(node);
            return true;
        }
        else if (!this.subGroups_) {
            if (fallBack)
                isScript ? this.scripts_.push(node) : this.nodes_.push(node);
            return fallBack;
        }
        return this.addToSubGroup(node, fallBack);
    }
    addToSubGroup(node, fallBack) {
        if (!this.subGroups_)
            return false;
        for (const g of this.subGroups_)
            if (g.add(node, fallBack))
                return true;
        return false;
    }
    /**
     * Sort imports within this group.
     *
     * 1. If `eslintGroupOrder_` is undefined, then sort `nodes_` and `subGroups_`
     *    based on user config.
     * 2. Otherwise:
     *   - If this is the top group (`level === 0`), then:
     *     - Sort `nodes_` (fallback group) based on ESLint and user config.
     *     - Sort `subGroups_` (user config groups) based on their rules.
     *   - If this is the user config group (`level === 1`), then:
     *     - Disregard `subGroups_`.
     *     - Merge all imports together and sort them based on ESLint and user config.
     *   - Else, no sort is needed.
     */
    sort(level = 0) {
        const { nodes_, sorter_, sortImportsBy_, eslintGroupOrder_: flags, aliasFirst_ } = this;
        if (flags && level > 1)
            return this;
        const byPaths = sortImportsBy_ != 'names';
        this.nodes_ = flags
            ? sortNodesByFlags(level === 0 ? nodes_ : this.allNodes_, flags, byPaths, sorter_, aliasFirst_)
            : (0, merge_1.sortImportNodes)(nodes_, byPaths, sorter_, aliasFirst_);
        if (flags && level === 1) {
            this.scripts_ = [];
            this.ignoreSubGroups_ = true;
        }
        else
            this.subGroups_?.forEach(g => g.sort(level + 1));
        return this;
    }
    compose(config, sep) {
        const { nl } = config;
        const subGroups = this.ignoreSubGroups_
            ? []
            : this.subGroups_?.map(g => g.compose(config, nl)) ?? [];
        return [
            this.scripts_.map(n => n.compose(config)).join(nl),
            ...subGroups,
            this.nodes_.map(n => n.compose(config)).join(nl),
        ]
            .filter(t => !!t)
            .join(sep);
    }
    calcSubGroups(groupRules, flags, eslint) {
        return groupRules
            ?.map(r => {
            return typeof r === 'string' ? { regex: r } : Array.isArray(r) ? { subGroups: r } : r;
        })
            .map(r => new SortGroup(r, { sorter: this.sorter_, flags, sortImportsBy: this.sortImportsBy_ }, eslint));
    }
    get allNodes_() {
        return (this.subGroups_ ?? []).reduce((r, g) => [...r, ...g.allNodes_], [...this.scripts_, ...this.nodes_]);
    }
    /**
     * Infer flags (step 1) from current config (flags) and parent's flags.
     */
    static inferFlags1(flags, parentFlags) {
        const bp = breakFlags(parentFlags);
        return flags.length > 0 ? flags : removeScriptsIfCombined(bp);
    }
    /**
     * Infer flags (step 2) from children's flags.
     */
    static inferFlags2(flags, flags1, subGroups) {
        if (flags.length > 0)
            return flags;
        const childrenFlags = subGroups?.reduce((r, g) => [...r, ...g.flags_], []);
        return childrenFlags && childrenFlags.length > 0 ? dedupFlags(childrenFlags) : flags1;
    }
}
exports.default = SortGroup;
function dedupFlags(flags) {
    return [...new Set(flags)];
}
function breakFlags(flags) {
    if (!flags)
        return [];
    const f = typeof flags === 'string' ? [flags] : flags;
    return dedupFlags(f.reduce((r, f) => [...r, ...(0, config_1.breakFlag)(f)], []));
}
function removeScriptsIfCombined(flags) {
    return flags.length > 1 ? flags.filter(f => f !== 'scripts') : flags;
}
function isFlagIncluded(flag, flags) {
    return flags.some(f => f === flag);
}
/**
 * First group `nodes` in the order defined by `flags`, then sort nodes inside each group.
 * Please note `flags` must be ATOMIC, i.e. neither 'all' nor 'named'.
 */
function sortNodesByFlags(nodes, flags, byPath, sorter, aliasFirst) {
    const order = flags.map(f => ({ flag: f, nodes: new Array() }));
    const fallback = [];
    nodes.forEach(n => (order.find(g => g.flag === n.flagType)?.nodes ?? fallback).push(n));
    const sort = (nodes) => (0, merge_1.sortImportNodes)(nodes, byPath, sorter, aliasFirst);
    const groups = [
        ...order.map(({ flag, nodes }) => (flag === 'scripts' ? nodes : sort(nodes))),
        sort(fallback),
    ];
    return groups.reduce((r, g) => [...r, ...(0, merge_1.sortImportNodes)(g, byPath, sorter, aliasFirst)], []);
}
//# sourceMappingURL=SortGroup.js.map