"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compareBindingName = exports.compareImportNodesByNames = exports.compareImportNodesByPaths = exports.updateSorterWithRules = exports.sorterFromRules = void 0;
const segment_sort_1 = __importDefault(require("segment-sort"));
const config_1 = require("../../config");
function sorterFromRules(rules) {
    const r = rules === 'none' ? { paths: 'none', names: 'none' } : rules;
    return {
        ...r,
        comparePaths: comparatorFromRule(r?.paths),
        compareNames: comparatorFromRule(r?.names),
    };
}
exports.sorterFromRules = sorterFromRules;
function updateSorterWithRules(sorter, rules) {
    if (!rules)
        return sorter;
    const r = { ...sorter };
    const { paths, names } = rules === 'none' ? { paths: 'none', names: 'none' } : rules;
    if (paths && paths !== r.paths) {
        r.paths = paths;
        r.comparePaths = comparatorFromRule(paths);
    }
    if (names && names !== r.names) {
        r.names = names;
        r.compareNames = comparatorFromRule(names);
    }
    return r;
}
exports.updateSorterWithRules = updateSorterWithRules;
function compareImportNodesByPaths(a, b, comparePaths, compareNames) {
    return (comparePaths(a.moduleIdentifier, b.moduleIdentifier) ||
        compareTypeOnly(a.isTypeOnly, b.isTypeOnly) ||
        (compareNames
            ? compareDefaultName(a.defaultName, b.defaultName, compareNames) ||
                compareBinding(a.binding, b.binding, compareNames, false)
            : 0));
}
exports.compareImportNodesByPaths = compareImportNodesByPaths;
function compareImportNodesByNames(a, b, comparePaths, compareNames, aliasFirst) {
    return (compareNameArrays(a.allNames(), b.allNames(), compareNames) ||
        compareTypeOnly(a.isTypeOnly, b.isTypeOnly) ||
        compareDefaultName(a.defaultName, b.defaultName, compareNames) ||
        compareBinding(a.binding, b.binding, compareNames, aliasFirst) ||
        (comparePaths ? comparePaths(a.moduleIdentifier, b.moduleIdentifier) : 0));
}
exports.compareImportNodesByNames = compareImportNodesByNames;
function compareNameArrays(n1, n2, compareNames) {
    let i = 0;
    for (; i < n1.length && i < n2.length; ++i) {
        const r = compareNames(n1[i], n2[i]);
        if (r !== 0)
            return r;
    }
    return i < n1.length ? 1 : i < n2.length ? -1 : 0;
}
/**
 * Sort typed imports in front of other imports.
 */
function compareTypeOnly(a, b) {
    return (a ? 0 : 1) - (b ? 0 : 1);
}
/**
 * Put default import in front of binding imports to highlight, e.g.:
 *
 * ```ts
 * import D from './a';         // comment to disable merge
 * import * as A from './a';    // namespace binding import
 * import { B, C } from './a';  // named binding import
 * ```
 */
function compareDefaultName(a, b, compare) {
    if (!a)
        return b ? 1 : 0;
    if (!b)
        return -1;
    return compare(a, b);
}
/**
 * Compare bindings.
 *
 * Note that namespace binding is sorted in front of named bindings, e.g.:
 *
 * ```ts
 * import * as C from './a';    // namespace binding import
 * import { A, B } from './a';  // named binding import
 * ```
 */
function compareBinding(a, b, compare, aliasFirst) {
    if (!a)
        return b ? -1 : 0;
    if (!b)
        return 1;
    if (a.type === 'named' && b.type === 'named')
        return compareBindingName(a.names[0], b.names[0], compare, aliasFirst);
    // Put namespace binding in front of named bindings.
    if (a.type === 'named')
        return 1;
    if (b.type === 'named')
        return -1;
    return compare(a.alias, b.alias);
}
function compareBindingName(a, b, compare, aliasFirst) {
    if (!a)
        return b ? -1 : 0;
    else if (!b)
        return 1;
    // 'default' < 'x as default' < 'default as x' < others
    if (isDefault(a))
        return isDefault(b) ? 0 : -1;
    else if (isDefault(b))
        return 1;
    const { propertyName: pa, aliasName: aa } = a;
    const { propertyName: pb, aliasName: ab } = b;
    if (isAsDefault(a))
        return isAsDefault(b) ? comparePropertyName(pa, pb, compare) : -1;
    else if (isAsDefault(b))
        return 1;
    return aliasFirst
        ? compare(aa ?? pa, ab ?? pb) || comparePropertyName(pa, pb, compare)
        : comparePropertyName(pa, pb, compare) || compare(aa, ab);
}
exports.compareBindingName = compareBindingName;
function isDefault(a) {
    return a.propertyName === 'default' && !a.aliasName;
}
function isAsDefault(a) {
    return !!a.propertyName && a.aliasName === 'default';
}
/**
 * Compare propertyName of named bindings.
 *
 * Note that `default as X` is sorted in front of other names to highlight, e.g.:
 *
 * ```ts
 * import { default as C, A, B } from './a';
 * ```
 */
function comparePropertyName(a, b, compare) {
    if (!a)
        return b ? -1 : 0;
    if (!b)
        return 1;
    // Put 'default as X' in front of any other binding names to highlight.
    if (a === 'default')
        return b === 'default' ? 0 : -1;
    if (b === 'default')
        return 1;
    return compare(a, b);
}
// Default comparator
const COMPARE_DEF = (a, b) => {
    if (!a)
        return !b ? 0 : -1;
    if (!b)
        return 1;
    return a < b ? -1 : a > b ? 1 : 0;
};
function comparatorFromRule(rule) {
    if (rule === null)
        return COMPARE_DEF;
    const r = rule === undefined ? config_1.COMPARE_RULE_DEFAULT : rule;
    if (r === 'none')
        return undefined;
    return (0, segment_sort_1.default)(r) ?? COMPARE_DEF;
}
//# sourceMappingURL=compare.js.map